/*
*  RemailerDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
#define _WINSOCKAPI_      // Prevent winsock.h from being included
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <sysutils.hpp>
#include <filectrl.hpp>
#pragma hdrstop

#include "RemailerDlgs.h"
#include "MixKeyWins2.h"
#include "MixProbWins2.h"
#include "CpkKeyWins2.h"
#include "HttpProxyDlgs.h"
#include "BarredRemsDlgs.h"
#include "Global.h"
#include "FileMan.h"
#include "UrlsDlgs.h"
#include "Mixmaster.h"
#include "Cyberpunk.h"
#include "PgpMail.h"
#include "Pgpfiles.h"
#include "NymMail.h"
#include "HelpDlgs.h"
#include "docmsgdlg.h"
#include "smtpdlgs.h"
#include "deldocsdlg.h"
#include "view.h"
#include "dntaskdlgs.h"
//---------------------------------------------------------------------------
#pragma link "HttpProt"
#pragma link "explbtn"
#pragma resource "*.dfm"
TRemailerDlg *RemailerDlg;
const AnsiString ChecksPage = "Error Checks";
//---------------------------------------------------------------------------
bool __fastcall GetRemailerStats(TForm *Owner, bool attended)
{
   RemailerDlg = new TRemailerDlg(Owner);
   bool ok = RemailerDlg->Execute(attended);
   if (ok) RegisterStatsUpdated();
   delete RemailerDlg;
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall DocumentErrorChecking()
{
   return GetAppIni(StatsErrCheck,"").ToIntDef(1);
}
//---------------------------------------------------------------------------
void __fastcall RegisterStatsUpdated()
{
   AppMainForm->Perform(WM_UPDATECHECKED, 1, NULL);
}
//---------------------------------------------------------------------------

__fastcall TRemailerDlg::TRemailerDlg(TComponent* Owner)
    : TForm(Owner)
{
   Cap = Caption;
   initcomplete = false;
   CenterDialog(this, Owner);
 ((TForm*)Owner)->Perform(WM_SHOWSTATUS,0,(LPARAM)"");
   if (FileExists(clisttmp)) DeleteFile(s(clisttmp).c_str());
   ErrorCheck->Checked = DocumentErrorChecking();
   TimeOut = GetAppIni(StatsTimeOut,"").ToIntDef(30);

   Label1->Caption = "";
   PageControl1->ActivePage = TabSheet1;

   if (!HasCpunk() && !HasNyms())
   {
      CRingBox->Checked = false;
      CRingBox->Visible = false;
      CRingCBox->Visible = false;
      Label3->Top -= 25;
   }

   Tabs = TabSet->Tabs;
   CancelAnyway = false;
   Cancelled = false;
   Attended = false;

   HttpCli1->RcvdStream = new TMemoryStream();
   CheckDeleteBarredMixmasterKeys();
   Timer3->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::Timer3Timer(TObject *Sender)
{
   Timer3->Enabled = false;
   Cursor = crHourGlass;

   LoadHttpInfo();

   MixSelfUpdate = false;
   CpkSelfUpdate = false;

   mixnews = new TMixKeyList2();
   mixdups = new TMixKeyList2();
   mixnokey = new TStringList();
   mixnokey->Sorted = true;
   mixkeys = NULL;

   cpknews = NULL;
   cpkdups = NULL;
   cpkkeys = NULL;
   cpknokey = NULL;
   if (HasCpunk())
   {
      cpknews = new TCpkKeyList2();
      cpkdups = new TCpkKeyList2();
      cpknokey = new TStringList();
      cpknokey->Sorted = true;
   }

   summary = new TStringList();
   MixGenType2();
   initcomplete = true;
   
   if (!Attended)
   {
      Cursor = crDefault;
      return;
   }

 ((TForm*)Owner)->Perform(WM_SHOWSTATUS,0,(LPARAM)"");
   if (Tabs->Count) TabSet->TabIndex = 0;
   Cursor = crDefault;
}
//---------------------------------------------------------------------------
__fastcall TRemailerDlg::~TRemailerDlg()
{
   for (int i=0; i<Tabs->Count; i++)
      delete (TRichEdit*)Tabs->Objects[i];
   delete HttpCli1->RcvdStream;
   delete summary;

   if (mixkeys) delete mixkeys;
   delete mixnokey;
   delete mixdups;
   delete mixnews;

   if (HasCpunk())
   {
      if (cpkkeys) delete cpkkeys;
      delete cpknokey;
      delete cpkdups;
      delete cpknews;
   }
}
//---------------------------------------------------------------------------
TRichEdit* __fastcall TRemailerDlg::GetRemailerDoc(int Index)
{
   if (Index==-2) Index = TabSet->TabIndex;
   if (Index<0 || Index>=Tabs->Count) return NULL;
   TRichEdit *doc=(TRichEdit*)Tabs->Objects[Index];
   if (doc) return doc;
   return NULL;
}
//---------------------------------------------------------------------------
TRichEdit* __fastcall TRemailerDlg::GetRemailerDoc(AnsiString DocName)
{
   int Index=Tabs->IndexOf(DocName);
   return GetRemailerDoc(Index);
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::RemoveDoc(int Index)
{
   if (Index<0 || Index>=Tabs->Count) return false;
   delete (TRichEdit*)Tabs->Objects[Index];
   Tabs->Delete(Index);
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::RemoveDoc(AnsiString DocName)
{
   return RemoveDoc(Tabs->IndexOf(DocName));
}
//---------------------------------------------------------------------------
TRichEdit* __fastcall TRemailerDlg::CreateDoc(TStrings *lines)
{
   TRichEdit *doc = new TRichEdit(this);
   doc->MaxLength = 1024000;
   doc->Parent = ResultsSheet;
   doc->Align = alClient;
   doc->ScrollBars = ssBoth;
   doc->HideScrollBars = false;
   doc->Font->Name = "Courier New";
   doc->Font->Size = 10;
   doc->WordWrap = false;
   doc->PlainText = true;
   doc->Lines->AddStrings(lines);
   return doc;
}
//---------------------------------------------------------------------------
TRichEdit* __fastcall TRemailerDlg::CreateDocHidden()
{
   TRichEdit *doc = new TRichEdit(this);
   doc->MaxLength = 1024000;
   doc->Visible = false;
   doc->Parent = ResultsSheet;
   doc->Align = alClient;
   doc->ScrollBars = ssBoth;
   doc->HideScrollBars = false;
   doc->Font->Name = "Courier New";
   doc->Font->Size = 10;
   doc->WordWrap = false;
   doc->PlainText = true;
   return doc;
}
//---------------------------------------------------------------------------
TRichEdit* __fastcall TRemailerDlg::CreateDoc(AnsiString Filename)
{
   TStringList *lines = new TStringList();
   if (FileExists(Filename)) lines->LoadFromFile(Filename);
   TRichEdit *doc=CreateDoc(lines);
   delete lines;
   return doc;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::WMGetButtonClick(TMessage& Msg)
{
// ClearEverything();
   GetBtnClick();
}
//---------------------------------------------------------------------------
void __fastcall SetCBoxCur(TComboBox *cbox, int listpos)
{
   if (listpos<cbox->Items->Count) cbox->ItemIndex = listpos;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::LoadURLs()
{
   GetAppData(MListCBox->Items,StatsURLs+PS+MListTxt); SetCBoxCur(MListCBox,0);
   GetAppData(MRingCBox->Items,StatsURLs+PS+MRingTxt); SetCBoxCur(MRingCBox,0);
   GetAppData(MTyp2CBox->Items,StatsURLs+PS+MTyp2Txt); SetCBoxCur(MTyp2CBox,0);
   GetAppData(CListCBox->Items,StatsURLs+PS+CListTxt); SetCBoxCur(CListCBox,0);
   if (HasCpunk())
   {
      GetAppData(CRingCBox->Items,StatsURLs+PS+CRingTxt); SetCBoxCur(CRingCBox,0);
    //CListCBox->Items->Add(""); SetCBoxCur(CListCBox,0);
      CRingCBox->Items->Add(""); SetCBoxCur(CRingCBox,0);
   }
   GetAppURLs();
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::SaveErrChkLog(TStrings *Log)
{
   if (!Log || !Log->Count) return;
   TStringList *list = new TStringList();
   GetAppData(list,ErrChkLog);
   int max = GetAppIni(ErrChkLogMax,"").ToIntDef(16);
   while (max && list->Count>=max)
   {
      PutAppData(ErrChkLog+PS+list->Strings[0],NULL); // delete existing log 0
      list->Delete(0);                                // delete from list
   }
   AnsiString logstr = SMTPPOP3logtime()+" "+Log->Strings[Log->Count-1];
   PutAppData(ErrChkLog+PS+logstr,Log);
   delete list;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::GetLastErrChkLog(TStrings *Log)
{
   Log->Clear();
   TStringList *list = new TStringList();
   GetAppData(list,ErrChkLog);
   if (list->Count)
      GetAppData(Log,ErrChkLog+PS+list->Strings[list->Count-1]);
   delete list;
   return Log->Count>0;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   switch (ModalResult)
   {
      case mrOk: SaveStatistics(); break;
      case mrCancel:
         if (GoodTabs())
         {
            switch (MsgBoxHwnd(Handle,
               "Your remailer docs are not saved!\n\nSave them now?",
               "Remailer Documents",MB_YESNOCANCEL))
            {
               case ID_CANCEL:
                  CanClose = false;
                  break;
               case ID_YES:
                  SaveStatistics();
                  break;
            }
         }
   }
   if (!CanClose) return;

   if (CancelAnyway) ModalResult = mrCancel;
   PutAppIni(StatsErrCheck,ErrorCheck->Checked ? "1":"0");
   if (FileExists(clisttmp)) DeleteFile(s(clisttmp).c_str());
   StoreHttpInfo();
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::CheckDocumentsExist()
{
   int reply;
   if (!Cancelled && !FileExists(mlisttxt) && MRingBox->Checked && !MListBox->Checked)
   {
      reply = MsgBoxHwnd(Handle,
        "Your 'mlist.txt' document is missing. Shall I get it while online?",
        "Mixmaster Statistics",
         MB_YESNOCANCEL
      );
      switch (reply)
      {
         case IDCANCEL:
            Cancelled = true;
            break;
         case IDYES:
            MListBox->Checked = true;
            break;
      }
   }
   MixGenType2();
   if (Cancelled) return;

   if (HasCpunk() && !FileExists(clisttxt) && CRingBox->Checked && !CListBox->Checked)
   {
      reply = MsgBoxHwnd(Handle,
        "Your 'rlist.txt' document is missing. Shall I get it while online?",
        "Cyberpunk Statistics",
         MB_YESNOCANCEL
      );
      switch (reply)
      {
         case IDCANCEL:
            Cancelled = true;
            break;
         case IDYES:
            CListBox->Checked = true;
            break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::ClearDocuments()
{
   ResultsSheet->Caption = ""; //ResultsSheet->Caption = "clearing..."; Snooze(100);
   for (int i=0; i<Tabs->Count; i++)
      delete (TRichEdit*)Tabs->Objects[i];
   Tabs->Clear();
   ResultsSheet->Caption = "Results";
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::IsTab(int Index, AnsiString &TabName)
{
   if (Index>=0 && Index<Tabs->Count)
      return Tabs->Strings[Index]==TabName;
   return false;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::ResetSelfUpdate()
{
  if (MixSelfUpdate)
  {
     MRingBox->Checked = false;
     MixSelfUpdate = false;
  }
  if (CpkSelfUpdate)
  {
     CRingBox->Checked = false;
     CpkSelfUpdate = false;
  }
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::ReportLinesSpill()
{
   int index = TabSet->TabIndex;
   if (index>=0)
   {
      TRichEdit *ed = GetRemailerDoc(index);
      if (((ed->Lines->Count+1)*ed->Font->Size)<=ed->ClientRect.Bottom)
         return true;
   }
   return false;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::GetDocsClick(TObject *Sender)
{
   if (!MListBox->Checked && !MRingBox->Checked && !MTyp2Box->Checked &&
       !CListBox->Checked && !CRingBox->Checked)
   {
      MsgBoxHwnd(Handle,"Nothing checked for update!","Document Error",MB_OK);
      return;
   }
   if (!MListBox->Checked && !MRingBox->Checked && !MTyp2Box->Checked &&
       !CListBox->Checked && !CRingBox->Checked)
   {
      MsgBoxHwnd(Handle,"Nothing checked for update!","Document Error",MB_OK);
      return;
   }
   bool cancel=false;
   if (HasCpunk() && CRingBox->Checked && !CListBox->Checked) CListBox->Checked = true;
   GetAppURLs(); CheckDocumentsExist();
   if (GoodTabs())
   {
      switch (MsgBoxHwnd(Handle,
         "You have good remailer docs not saved!\n\nSave them now?",
         "Remailer Documents",MB_YESNOCANCEL))
      {
         case ID_YES:
            SaveStatistics();
            break;
         case ID_CANCEL:
            cancel = true;
            break;
      }
   }
   if (cancel) return;

   ClearDocuments();
   if (HasCpunk())
   {
      cpknokey->Clear();
      cpknews->Clear();
      cpkdups->Clear();
   }
   mixnokey->Clear();
   mixnews->Clear();
   mixdups->Clear();

   TabSet->Enabled = false;
   GetBtnClick();
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::GetBtnClick()
{
   if (!MListBox->Checked && !MRingBox->Checked && !MTyp2Box->Checked &&
       !CListBox->Checked && !CRingBox->Checked)
   {
      MsgBoxHwnd(Handle,"Nothing checked for update!","Document Error",MB_OK);
      Cancelled = true;
   }

   if (Cancelled)
   {
      TabSet->Enabled = true;
      ResetSelfUpdate();
      ResetCheckboxes();
      return;
   }

   if (!Timer1->Enabled)
      Timer1->Enabled = true;

   if (GetDocs->Enabled && GetDocs->Caption=="Get Docs")
      Panel3->Enabled = false;

   GetDocs->Enabled = false;
   HelpBtn->Enabled = false;
   OkBtn->Enabled = false;

   if (MListBox->Enabled && MListBox->Checked) GetStatistics(MListURL,MListBox,MListTxt); else
   if (MRingBox->Enabled && MRingBox->Checked) GetStatistics(MRingURL,MRingBox,MRingTxt); else
   if (MTyp2Box->Enabled && MTyp2Box->Checked) GetStatistics(MTyp2URL,MTyp2Box,MTyp2Txt); else
   if (CListBox->Enabled && CListBox->Checked) GetStatistics(CListURL,CListBox,CListTxt); else
   if (CRingBox->Enabled && CRingBox->Checked) GetStatistics(CRingURL,CRingBox,CRingTxt);
   else
   {
      ResetCheckboxes(); Label1->Caption = "";
      ProgressBar1->Visible = false; Label1->Caption = "";
      TabSet->TabIndex = Tabs->Count-1;
      if (ErrorCheck->Checked)
      {
         Sleep(200);
         TRichEdit *Doc;
         Tabs->AddObject(ChecksPage,CreateDoc(""));
         TabSet->TabIndex = Tabs->Count-1;
         Application->ProcessMessages();
         bool ok1=true,ok2=true,ok3=true,ok4=true;

         for (int i=0; i<Tabs->Count; i++)
         {
            if (Cancelled) break;
            if ((Doc=dynamic_cast<TRichEdit*>(Tabs->Objects[i]))!=NULL)
            {
               Doc->Modified = true;
               if (IsTab(i,MListTxt)) ok1=MixListOk(Doc->Lines); else
               if (IsTab(i,MRingTxt)) ok2=MixRingOk(Doc->Lines); else
               if (IsTab(i,CListTxt)) ok3=CpkListOk(Doc->Lines); else
               if (IsTab(i,CRingTxt)) ok4=CpkRingOk(Doc->Lines);
            }
         }

         if (!ok1 || !ok2 || !ok3 || !ok4 || Cancelled)
         {
            if (!Attended) Display("Operation cancelled");
            Display("Done!");
            if (TabSet->TabIndex>=0)
            {
               TRichEdit *doc=GetRemailerDoc(TabSet->TabIndex);
               if (doc && doc->Lines->Text.Pos("Failed Error Check")>0)
               {
                  DontShowFailedErrorCheck(this);
               }
            }

            TabSet->Enabled = true;
            CancelAnyway = true;
            Cancelled = true;
            Attended = true;

            GetDocs->Enabled = true;
            HelpBtn->Enabled = true;
            OkBtn->Enabled = true;

            ResetSelfUpdate();
            ResetCheckboxes();
            //GetBtnClick();
            return;
         }

         TabSet->Enabled = true;
         AppProcessMessages();
         Sleep(100);
      }

      ResetSelfUpdate();
      AppProcessMessages(); Sleep(100);
      Panel3->Enabled = true;
      UrlsBtn->Enabled = true;
      ProxyBtn->Enabled = true;
      GetDocs->Enabled = true;
      OkBtn->Enabled = true;
      HelpBtn->Enabled = true;
      Label1->Caption = "";
      Cancelled = false;

      if (Attended) MyBeep(MB_OK); // if attended, just this line
      else
      {
         ModalResult = !Cancelled || (Cancelled && Tabs->Count>1) ? mrOk : mrCancel;
         ProgressBar1->Visible = false;
         CancelBtn->Enabled = false;
         GetDocs->Enabled = false;
         OkBtn->Enabled = false;
         HelpBtn->Enabled = false;
         AppProcessMessages();
         Sleep(100);
      }
   }
}
//---------------------------------------------------------------------------
/*void __fastcall ClearTypeList(TStringList *TypeList)
{  // clears type2.list of all but disables (#)
   int t=0; while (t<TypeList->Count)
   {
      if (TypeList->Strings[t].Pos("#")!=1) TypeList->Delete(t);
      else t++;
   }
}*/
//---------------------------------------------------------------------------
TRichEdit* __fastcall TRemailerDlg::GetDocument(AnsiString &DocName)
{
   TRichEdit *Doc;
   // if the doc was not downloaded get it from disk
   int Index=Tabs->IndexOf(DocName);
   if (Index>=0) Doc=(TRichEdit*)Tabs->Objects[Index]; // is it already open
   else
   {
      Doc = CreateDoc(MixDir()+DocName); // ok, make a new one
      Tabs->AddObject(DocName,Doc);
      TabSet->TabIndex = Tabs->Count-1;
      AppProcessMessages(); Sleep(200);
   }
   // special handling cpunk keyring
   //if (DocName==CRingTxt && HasCpunk()) CpkBlockToKeysFull(Doc->Lines);
   // end special
   return Doc;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::CorruptDoc(TStrings *lines)
{
   return lines && lines->Text.Trim().Pos(CorruptReport)==1;
}
//---------------------------------------------------------------------------
bool __fastcall GetLongEQShorts(TStrings *LEQS, TStrings *rlist)
{
   int c=0;
   for (int i=0; i<rlist->Count; i++)
   {
       AnsiString shrt,addr,line=rlist->Strings[i];
       if (line.Pos("$remailer{\"")==1)
       {
          shrt = GetToken(1,line,"\"");
          addr = GetToken(0,GetToken(1,line,"<"),">");
          if (LEQS->Values[addr].IsEmpty())
          {
             LEQS->Values[addr] = shrt;
             c++;
          }
       }
   }
   return c>0;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::SaveStatistics()
{
   if (!GoodTabs()) return false;

   TRichEdit *TxtList;
   AnsiString TxtFile,name;
   int i,pos,bakcopies=STATSBAKCOPIES;
   for (i=0; i<Tabs->Count; i++)
   {
      TxtFile = Tabs->Strings[i];
      TxtList = dynamic_cast<TRichEdit*>(Tabs->Objects[i]);
      if (!TxtList->Modified || TxtList->Text.IsEmpty() || TxtFile=="Error" ||
           TxtFile==MTyp2Txt || TxtFile==CTyp1Txt ||
           TxtList->Text.Pos(CorruptReport)==1)
         continue;

      if (TxtFile==ChecksPage && !TxtList->ReadOnly)
      {
         SaveErrChkLog(TxtList->Lines);
         continue;
      }

      if (TxtFile==MListTxt)
      {
         SaveToFileBak(TxtList->Lines,MixDir()+TxtFile,bakcopies);
         continue;
      }

      if (TxtFile==CListTxt)
      {
         TStringList *LEQS = new TStringList(); // handle the longs EQ shorts list
         GetAppData(LEQS,CpunkShorts);
         if (GetLongEQShorts(LEQS,TxtList->Lines))
         {  // if it's my computer save to masterlist file
            AnsiString masterlist = AppDir+"ini\\"+CpunkShorts+".txt";
            if (ISDESIGNING && FileExists(masterlist))
            {
               // get list from disk and add new items;
               TStringList *tmps = new TStringList();
               tmps->LoadFromFile(masterlist);
               for (int i=0; i<tmps->Count; i++)
               {
                  name = tmps->Names[i];
                  LEQS->Values[name] = tmps->Values[name];
               }
               LEQS->SaveToFile(masterlist);
               delete tmps;
            }
            PutAppData(CpunkShorts,LEQS);
         }
         SaveToFileBak(TxtList->Lines,MixDir()+TxtFile,bakcopies);
         delete LEQS;
         continue;
      }

      if (TxtFile==MRingTxt)
      {
         for (int i=0; i<mixdups->Count; i++)
         {
            if (mixnews->Find(mixdups->Strings[i],pos))
            {
                mixnews->Delete(pos);
            }
         }
         if (mixnews->Count)
         {
            if (!MixKeyWin2) MixKeyWin2 = new TMixKeyWin2(AppMainForm);
            else MixKeyWin2->WindowState = wsNormal;
            MixKeyWin2->BringToFront();
            MixKeyWin2->AddKeysImport(mixnews);
            AnsiString tmp=FormatDateTime("yyyy-mm-dd",Date());
         }
         if (mixdups->Count)
         {
            if (!MixProbWin2)
            {
               MixProbWin2 = new TMixProbWin2(AppMainForm,mixdups);
               MixProbWin2->probfile = mprobtxt;
            }
            else MixProbWin2->WindowState = wsNormal;
            MixProbWin2->BringToFront();
            MixProbWin2->AddKeysImport(mixdups);
         }
         PutAppIni(MixKeyUpdate,FormatDateTime("yyyy-mm-dd",Date()));
         continue;
      }

      if (TxtFile==CRingTxt)
      {
         if (cpknews->Count)
         {
            for (int i=0; i<cpkdups->Count; i++)
            {
               if ((pos=cpknews->IndexOf(cpkdups->Strings[i]))>=0)
                  cpknews->Delete(pos);
            }
            if (!CpkKeyWin2) CpkKeyWin2 = new TCpkKeyWin2(AppMainForm);
            else CpkKeyWin2->WindowState = wsNormal;
            AppProcessMessages();
            CpkKeyWin2->BringToFront();
            CpkKeyWin2->AddKeysImport(cpknews);
         }
         if (cpkdups->Count)
         {
            if (!CpkKeyWin2) CpkKeyWin2 = new TCpkKeyWin2(AppMainForm);
            else CpkKeyWin2->WindowState = wsNormal;
            AppProcessMessages();
            CpkKeyWin2->BringToFront();
            CpkKeyWin2->AddDupsImport(cpkdups);
         }
         PutAppIni(CpkKeyUpdate,FormatDateTime("yyyy-mm-dd",Date()));
         continue;
      }
   }
   return true;
}
//---------------------------------------------------------------------------
int __fastcall TRemailerDlg::GoodTabs()
{
   int valtabs=0;
   TRichEdit *doc;
   for (int i=0; i<Tabs->Count; i++)
   {
      if (Tabs->Strings[i]=="Error" || Tabs->Strings[i]==ChecksPage) continue;
      if ((doc=dynamic_cast<TRichEdit*>(Tabs->Objects[i]))!=NULL)
      {
         if (doc->Lines->Text.Pos(CorruptReport)==1) continue;
         if (doc->Modified) valtabs++;
      }
   }
   return valtabs;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::FormShow(TObject *Sender)
{
//sm("starting dialog show form");
   ActiveControl = Panel2; LoadURLs();
   if (!Attended) PostMessage(Handle,WM_GETBUTTONCLICK,0,NULL);
//sm("end dialog show form");
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::Display(AnsiString &text)
{
   int Index = TabSet->TabIndex;
   if (Index>=0)
   {
      TRichEdit *doc=GetRemailerDoc(Index);
      if (doc)
      {
         ActiveControl = doc;
         doc->Lines->Add(text);
         AppProcessMessages();
         Sleep(10);
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::Summary(AnsiString &text)
{
   summary->Add(text);
}
//---------------------------------------------------------------------------
AnsiString KeyRingCnt(int val)
{
   AnsiString v=AnsiString(val);
   if (v.Length()<2) v = " "+v; return v;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::CpkListOk(TStrings *Doc)
{
//sm("Checking rlist.txt");
   if (!HasCpunk()) return true;

   bool hasstatshdr = false;
   AnsiString line,dshline = "--------------------------------------------------------";
   //for(int i=0; i<Doc->Count; i++)
   //{
   //   line = Doc->Strings[i];
   //   while ((pos=line.Pos("&lt;"))>0 || (pos=line.Pos("&gt;"))>0)
   //      line.Delete(pos,4);
   //   if (line!=Doc->Strings[i]) Doc->Strings[i] = line;
   //}
   // search for stats headers
   for (int i=0; i<Doc->Count; i++)
   {
      if (Doc->Strings[i].Pos(dshline)==1 && line.Pos("remailer ")==1)
      {
         hasstatshdr = true;
         break;
      }
      line = Doc->Strings[i];
   }
   if (!hasstatshdr)
   {
      AppProcessMessages();
      Doc->Insert(0,CorruptReport);
      Display("rlist.txt...Failed Error Check");
      Summary("rlist.txt...Failed Error Check");
      Display(" The new 'rlist.txt' document appears invalid. I'm");
      Display(" unable to locate these statistics column headings.");
      Display(" 'remailer  email address        history  latency  uptime'");
      Display(" The document has been marked so you can't accidentally");
      Display(" save it. See Help");
      AppProcessMessages(); Sleep(500);
      return false;
   }
   // check each line
   int c=0; for (int i=0; i<Doc->Count; i++)
      if (CpkListLineOk(Doc->Strings[i])) c++;
   if (!c)
   {
      AppProcessMessages();
      Doc->Insert(0,CorruptReport);
      Display("rlist.txt...Failed Error Check");
      Summary("rlist.txt...Failed Error Check");
      Display(" The new 'rlist.txt' document appears invalid. I've");
      Display(" located the stats column headers, but found no actual");
      Display(" stats lines at all. The document has been marked so");
      Display(" you can't accidentally save it. See Help");
      AppProcessMessages(); Sleep(500);
      return false;
   }
   // check for new remailers in list the we have no key for
   if (cpknokey->Count)
   {
      AnsiString rems,rem,theS = cpknokey->Count>1 ? "S":"";
      Display("rlist.txt..."+s(cpknokey->Count)+" NEW REMAILER"+theS+" Auto-updating pubring.asc");
      Summary("rlist.txt..."+s(cpknokey->Count)+" NEW REMAILER"+theS);
      int i=0; while (i<cpknokey->Count)
      {
         rems = "";
         while (rems.Length()<60 && i<cpknokey->Count)
         {
            rem = cpknokey->Strings[i++];
            rems += rem+s("             ").SubString(1,12-rem.Length());
         }
         Display("  "+rems);
      }
   }
   // remove invalid entries--for example '=znar'
   int i=0; while (i<cpkshorts->Count)
   {
      if (cpkshorts->Strings[i].Pos("=")==1) cpkshorts->Delete(i);
      else i++;
   }
   // gather new cpkshorts
   AnsiString shrt,addr;
   for (int i=0; i<Doc->Count; i++)
   {
      line = Doc->Strings[i];
      if (line.Pos("$remailer{")==1)
      {
         shrt = GetToken(1,line,"\"");
       //addr = GetToken(0,GetToken(1,line,"<"),">");
         addr = GetEAddress(line);
         cpkshorts->Values[addr] = shrt;
      }
   }
   // all is well
   Display("rlist.txt...OK");
   Summary("rlist.txt...OK");
   Doc->SaveToFile(clisttmp);
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::CpkRingOk(TStrings *Doc)
{
//sm("Checking pubring.asc");
   if (!HasCpunk()) return true;

   int begs=0,ends=0;
   AnsiString desc,line;
   for (int i=0; i<Doc->Count; i++)
   {
      line=Doc->Strings[i].Trim();
      if (line==BeginPgpPubKeyBlock) begs++; else
      if (line==EndPgpPubKeyBlock) ends++;
   }
   if (begs==0 || begs!=ends)
   {
      AppProcessMessages();
      Doc->Insert(0,CorruptReport);
      Display("pubring.asc...Failed Error Check");
      Summary("pubring.asc...Failed Error Check");
      Display(" The new 'pubring.asc' document appears invalid. The");
      Display(" values below should BOTH be greater than 0 AND equal.");
      Display(" "+s(begs)+" "+BeginPgpPubKeyBlock+" lines");
      Display(" "+s(ends)+" "+EndPgpPubKeyBlock+" lines");
      Display(" The document has been marked so you can't accidentally");
      Display(" save it."),
      AppProcessMessages(); Sleep(500);
      return false;
   }
   if (!HasCpunk()) return true;

   // doc is ok, check for new keys
   TCpkKeyList2 *curdb = new TCpkKeyList2(cringtxt);
   TCpkKeyList2 *newdb = new TCpkKeyList2(Doc);
   for (int i=0; i<newdb->Count; i++)
   {
      if (!curdb->Find(newdb->Strings[i],NULL))
         cpknews->AddKey(newdb->Key(i));
   }
   delete newdb;
   // any duplicates...
   for (int i=0; i<cpknews->Count; i++)
   {
      for (int c=0; c<curdb->Count; c++)
      {
         if (cpknews->Key(i)->name==curdb->Key(c)->name &&
             cpknews->Key(i)->algo==curdb->Key(c)->algo)
         {
            cpkdups->AddKeys(cpknews->Key(i));
            break;
         }
      }
   }
   delete curdb;

   if (cpknews->Count)
   {
      AnsiString theS = cpknews->Count>1 ? "S":"";
      Display("pubring.asc..."+s(cpknews->Count)+" NEW KEY"+theS);
      Summary("pubring.asc..."+s(cpknews->Count)+" NEW KEY"+theS);
      for (int i=0; i<cpknews->Count; i++) Display("  "+cpknews->KeyDescription(i));
   }
   if (cpkdups->Count)
   {
      AnsiString theS = cpkdups->Count>1 ? "S":"";
      Display("pubring.asc..."+s(cpkdups->Count)+" DUPLICATE KEY"+theS);
      Summary("pubring.asc..."+s(cpkdups->Count)+" DUPLICATE KEY"+theS);
      for (int i=0; i<cpkdups->Count; i++) Display("  "+cpkdups->KeyDescription(i));
   }

   Display("pubring.asc...OK");
   Summary("pubring.asc...OK");
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::CpkListNewRemailer(TStrings *cpklist)
{
   if (!HasCpunk()) return false;

   cpknokey->Clear();
   AnsiString cline,shrtname;
   TStringList *barred = new TStringList();
   GetAppData(barred,BarredCypherpunk);
   if (!cpkkeys) cpkkeys = new TCpkKeyList2(cringtxt);
   for (int c=0; c<cpklist->Count; c++)
   {
      cline = cpklist->Strings[c];
      if (!CpkListLineOk(cline)) continue;

      bool found=false;
      shrtname = GetToken(0,cline," ").LowerCase();
      for (int i=0; i<cpkkeys->Count; i++)
      {
         if (shrtname==cpkkeys->Key(i)->name)
         {
            found = true;
            break;
         }
      }
      if (!found)
      {
         for (int i=0; i<barred->Count; i++)
         {
            if (shrtname==GetToken(0,barred->Strings[i]," "))
            {
               found = true;
               break;
            }
         }
      }
      if (!found)
      {
         cpknokey->Add(shrtname);
      }
   }
   delete barred;
   return cpknokey->Count>0;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::MixListNewRemailer(TStrings *mixlist)
{
   mixnokey->Clear();
   AnsiString mline,shrtname;
   TStringList *barred = new TStringList();
   GetAppData(barred,BarredMixmaster);

   // make sure 'lcs' is barred -- doesn't support pasted headers
   if (barred->IndexOf(LCSremailer) < 0)
   {
      barred->Add(LCSremailer);
      PutAppData(BarredMixmaster, barred);
   }

   if (!mixkeys) mixkeys = new TMixKeyList2(mringtxt);
   for (int m=0; m<mixlist->Count; m++)
   {
      mline = mixlist->Strings[m];
      if (!MixListLineOk(mline)) continue;

      bool found=false;
      shrtname = GetToken(0,mline," ").LowerCase();
      for (int i=0; i<mixkeys->Count; i++)
      {
         if (shrtname==mixkeys->Key(i)->name)
         {
            found = true;
            break;
         }
      }
      if (!found)
      {
         for (int i=0; i<barred->Count; i++)
         {
            if (shrtname==GetToken(0,barred->Strings[i]," "))
            {
               found = true;
               break;
            }
         }
      }
      if (!found)
      {
         mixnokey->Add(shrtname);
      }
   }
   return mixnokey->Count>0;
}
//---------------------------------------------------------------------------
/*bool __fastcall TRemailerDlg::MixListNewRemailer(TStrings *mixlist)
{
   AnsiString mline,mshort,tshort;
   TStringList *type2 = new TStringList();
   if (FileExists(mtyp2txt)) type2->LoadFromFile(mtyp2txt);
   TStringList *mring = new TStringList();
   if (FileExists(mringtxt)) mring->LoadFromFile(mringtxt);

   MixGenType2(type2,mring);
   mixnokey->Clear();
   for (int m=0; m<mixlist->Count; m++)
   {
      mline = mixlist->Strings[m];
      if (!MixListLineOk(mline)) continue;

      bool found=false;
      mshort = GetToken(0,mline," ").LowerCase();
      for (int t=0; t<type2->Count; t++)
      {  // cannot use InType2 -- working with short names only
         tshort = GetToken(0,type2->Strings[t]," ");
         if (mshort==tshort || "#"+mshort==tshort)
         {
            found = true;
            break;
         }
      }
      if (!found) mixnokey->Add(mshort);
   }
   delete type2;
   delete mring;
   return mixnokey->Count>0;
}*/
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::MixListOk(TStrings *Doc)
{
//sm("Checking mlist.txt");
   //AnsiString statsheader =
   //   s("mixmaster           history  latency  uptime\r\n")+
   //   s("--------------------------------------------\r\n");
   bool hashead=false;
   for (int i = 0; i < Doc->Count; i++)
   {
      if (i < Doc->Count-1 &&
          Doc->Strings[i  ].Pos("mixmaster           ") == 1 &&
          Doc->Strings[i+1].Pos("--------------------------------------------")==1)
      {
         hashead = true;
         break;
      }
   }
   if (!hashead)
   {
      Doc->Insert(0,CorruptReport);
      Display("mlist.txt...Failed Error Check");
      Summary("mlist.txt...Failed Error Check");
      Display(" The new 'mlist.txt' document appears invalid. I'm");
      Display(" unable to locate these statistics columns headings.");
      Display(" \"mixmaster           history  latency  uptime\"");
      Display(" The document has been marked so you can't accidentally");
      Display(" save it. See Help");
      AppProcessMessages(); Sleep(500);
      return false;
   }
   int c = 0;
   for (int i = 0; i < Doc->Count; i++)
      if (MixListLineOk(Doc->Strings[i])) c++;
   if (!c)
   {
      Doc->Insert(0,CorruptReport);
      Display("mlist.txt...Failed Error Check");
      Summary("mlist.txt...Failed Error Check");
      Display(" The new 'mlist.txt' document appears invalid. I've");
      Display(" located the stats column headers, but found no actual");
      Display(" stats lines at all. The document has been marked so");
      Display(" you can't accidentally save it. See Help");
      AppProcessMessages(); Sleep(500);
      return false;
   }
   if (mixnokey->Count)
   {
      AnsiString rems,rem,theS = mixnokey->Count>1 ? "S":"";
      Display("mlist.txt..."+s(mixnokey->Count)+" NEW REMAILER"+theS+" Auto-updating pubring.mix");
      Summary("mlist.txt..."+s(mixnokey->Count)+" NEW REMAILER"+theS);
      int i=0; while (i<mixnokey->Count)
      {
         rems = "";
         while (rems.Length()<60 && i<mixnokey->Count)
         {
            rem = mixnokey->Strings[i++];
            rems += rem+s("             ").SubString(1,12-rem.Length());
         }
         Display("  "+rems);
      }
   }

   Display("mlist.txt...OK");
   Summary("mlist.txt...OK");
   return true;
}
//---------------------------------------------------------------------------
/*bool __fastcall DisabledInType2(TStrings *type2, AnsiString &keydescs)
{
   AnsiString desc;
   bool disabled = false;
   TStringList *descs = new TStringList();
   descs->Text = keydescs;
   for (int d=0; d<descs->Count; d++)
   {
      disabled = false;
      desc = descs->Strings[d];
      for (int t=0; t<type2->Count; t++)
      {
         if ("#"+desc==type2->Strings[t])
         {
            disabled = true;
            break;
         }
      }
      if (!disabled) break;
   }
   delete descs;
   return disabled;
}
//---------------------------------------------------------------------------
AnsiString __fastcall ShortExistsInCurdb(TMixKeyList2 *curdb, AnsiString &desc)
{
   TStringList *keys = new TStringList();
   AnsiString descs,shrt=GetToken(0,desc,",").LowerCase();
   for (int i=0; i<curdb->Count; i++)
   {
      if (shrt==GetToken(0,curdb->Strings[i],",").LowerCase())
         keys->Add(curdb->Strings[i]);
   }
   descs = keys->Text;
   delete keys;
   return descs;
}*/
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::MixRingOk(TStrings *Doc)
{
//sm("Checking pubring.mix");
   int descrips=0,begs=0,ends=0;
   AnsiString existing,line,desc,curdesc,name;
   for (int i=0; i<Doc->Count; i++)
   {
      line = Doc->Strings[i];
      if (MixRingLineOk(line)) descrips++; else
      if (line==BeginMixKey) begs++; else
      if (line==EndMixKey) ends++;
   }
//sm("Checking line counts...");
   if (!descrips || descrips!=begs || begs!=ends)
   {
      AppProcessMessages();
      Doc->Insert(0,CorruptReport);
      Display("pubring.mix...Failed Error Check");
      Summary("pubring.mix...Failed Error Check");
      Display(" The new 'pubring.mix' document appears invalid. The");
      Display(" values below should be greater than 0 AND equal.");
      Display("  "+KeyRingCnt(descrips)+"  Mixmaster key description lines");
      Display("  "+KeyRingCnt(begs)+"  "+BeginMixKey+" lines");
      Display("  "+KeyRingCnt(ends)+"  "+EndMixKey+" lines");
      Display(" The document has been marked so you can't accidentally");
      Display(" save it.");
      AppProcessMessages(); Sleep(500);
      return false;
   }
   //sm("Checking for new keys...");
   // doc is ok, check for new keys
   TMixKeyList2 *curdb = new TMixKeyList2();
   if (MixKeyWin2) curdb->AddKeys(MixKeyWin2->keys);
   else if (FileExists(mringtxt)) curdb->LoadFromFile(mringtxt);
   //sm("Checking mix problems...");
   TMixKeyList2 *prodb = new TMixKeyList2(mprobtxt);
   if (MixProbWin2)
   {
      prodb->AddKeys(MixProbWin2->keys);
      MixProbWin2->probfile = mprobtxt;
   }
   else if (FileExists(mprobtxt)) prodb->LoadFromFile(mprobtxt);
   //sm("Checking new db...");
   TMixKeyList2 *newdb = new TMixKeyList2(Doc);
   for (int i=0; i<newdb->Count; i++)
   {
      curdesc = newdb->Strings[i];
      if (!curdb->Find(curdesc,NULL) && !prodb->Find(curdesc,NULL))
      {
         mixnews->AddKey(newdb->Key(i));
//sm(mixnews->KeyDescription(mixnews->Count-1));
//view(mixnews,s(mixnews->addrLen));
      }
   }
//view(mixnews,s(mixnews->addrLen));
   delete newdb;
   delete prodb;
//sm("Checking duplicates...");
   // duplicates?...
   if (!FileExists(mringtxt) && FileExists(mtyp2txt)) DeleteFile(s(mtyp2txt).c_str());
   TStringList *type2 = new TStringList(); type2->Sorted = true;
   if (FileExists(mtyp2txt)) type2->AddStrings(curdb->type2);//LoadFromFile(mtyp2txt);
   for (int i=0; i<mixnews->Count; i++)
   {
      curdesc = mixnews->Strings[i];
      if (type2->Find("#"+curdesc,NULL)) continue;

      name = mixnews->Key(i)->name;
      for (int t=0; t<type2->Count; t++)
      {
         if (type2->Strings[t].Pos(name+" ")==1)
         {
             mixdups->AddKey(mixnews->Key(i));
//sm(mixdups->KeyDescription(0));
//view(mixdups,s(mixdups->addrLen));
             break;
         }
      }
   }
   delete curdb;
   delete type2;
//sm("Generating type2.list...");
  (TStrings*)type2=GetRemailerDoc(MTyp2Txt)->Lines;
   MixGenType2(type2,Doc);
   MixSpaceType2(type2);

//sm("Displaying new keys...");
   if (mixnews->Count)
   {
      AnsiString theS = mixnews->Count>1 ? "S":"";
      Display("pubring.mix..."+s(mixnews->Count)+" NEW KEY"+theS);
      Summary("pubring.mix..."+s(mixnews->Count)+" NEW KEY"+theS);
      for (int i=0; i<mixnews->Count; i++) Display("  "+mixnews->KeyDescription(i));
//view(mixnews,s(mixnews->addrLen));
   }
//sm("Displaying duplicate keys...");
   if (mixdups->Count)
   {
      mixdups->shrtLen = mixnews->shrtLen;
      mixdups->addrLen = mixnews->addrLen;
      AnsiString theS = mixdups->Count>1 ? "S":"";
      Display("pubring.mix..."+s(mixdups->Count)+" DUPLICATE KEY"+theS);
      Summary("pubring.mix..."+s(mixdups->Count)+" DUPLICATE KEY"+theS);
      for (int i=0; i<mixdups->Count; i++) Display("  "+mixdups->KeyDescription(i));
//view(mixdups,s(mixdups->addrLen));
   }
//sm("Done with pubring.mix");
   Display("pubring.mix...OK");
   Summary("pubring.mix...OK");
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::GetStatistics(AnsiString &forURL, TCheckBox *Box,
   AnsiString &Doc)
{
   CurDoc = Doc;
   CurBox = Box;
   Cap = forURL;//+"...";
   Label1->Caption = Cap;

   HttpCli1->URL = forURL;
   HttpCli1->Proxy = ProxyHost;
   HttpCli1->ProxyPort = ProxyPort;
   HttpCli1->ProxyUsername = ProxyUser;
   HttpCli1->ProxyPassword = ProxyPswd;
 ((TMemoryStream*)HttpCli1->RcvdStream)->SetSize(0);
   HttpCli1->ModifiedSince = 0;
   ProgressBar1->Position = 0;
   ProgressBar1->Visible = true;
   TimeOutTimer->Enabled = true;
   BytesReceived = 0;
   CurTime = 0;

   try {
      HttpCli1->GetASync();
    //HttpCli1->HeadASync();
   }
   catch (...)
   {
       Panel3->Enabled = true;
       TRichEdit *doc=GetRemailerDoc(CurDoc);
       doc->Clear();
       doc->Lines->Add("Get Failed !");
       doc->Lines->Add("Head Failed !");
       doc->Lines->Add("Status = " + IntToStr(HttpCli1->StatusCode));
       doc->Lines->Add("Reason = " + HttpCli1->ReasonPhrase);
       Tabs->AddObject(CurDoc,doc);
       TabSet->TabIndex = Tabs->Count - 1;
       PageControl1->ActivePage = ResultsSheet;
       ProgressBar1->Visible = false;
       TimeOutTimer->Enabled = false;

       return false;
   }
   return !Cancelled;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::LoadHttpInfo()
{
//sm("LOADHTTP");
   TStringList *http = new TStringList();
   GetAppData(http,HttpInfo);
   MListBox->Checked = http->Values["MListChkd"] == "1";
   MRingBox->Checked = http->Values["MRingChkd"] == "1";
   MTyp2Box->Checked = http->Values["MTyp2Chkd"] == "1";
   CListBox->Checked = HasCpunk() && http->Values["CListChkd"] == "1";
   CRingBox->Checked = HasCpunk() && http->Values["CRingChkd"] == "1";
   ProxyHost = http->Values["ProxyHost"];
   ProxyPort = http->Values["ProxyPort"];
   ProxyPswd = http->Values["ProxyPswd"];
   ProxyUser = http->Values["ProxyUser"];
 //CpkLoadView->Checked = http->Values["CpkLoadView"] == "1";
   Http = http->Text; // keep for later compare to see if modified
   delete http;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::StoreHttpInfo()
{
   TStringList *http = new TStringList();
   GetAppData(http,HttpInfo);
   http->Values["MListChkd"] = MListBox->Checked?"1":"0";
   http->Values["MRingChkd"] = MRingBox->Checked?"1":"0";
   http->Values["MTyp2Chkd"] = MTyp2Box->Checked?"1":"0";
   http->Values["CListChkd"] = CListBox->Checked?"1":"0";
   http->Values["CRingChkd"] = CRingBox->Checked?"1":"0";
   //http->Values["CpkLoadView"] = CpkLoadView->Checked?"1":"0";
   if (Http!=http->Text) // test for modified
      PutAppData(HttpInfo,http);
   delete http;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::Execute(bool attended)
{
   while (!initcomplete && !Application->Terminated)
      Application->ProcessMessages();
      
   Attended = attended; ShowModal();
   return ModalResult == mrOk;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::ResetCheckboxes()
{
   Timer1->Enabled = false;

   if (MListBox->Checked) MListBox->Enabled = true;
   if (MRingBox->Checked) MRingBox->Enabled = true;
   if (MTyp2Box->Checked) MTyp2Box->Enabled = true;
   if (HasCpunk())
   {
      if (CListBox->Checked) CListBox->Enabled = true;
      if (CRingBox->Checked) CRingBox->Enabled = true;
   }
   Panel3->Enabled = true;
//TabSet->Enabled = true;
   Cancelled = false;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::HttpCli1DocBegin(TObject *Sender)
{
   BytesReceived = 0;
   ProgressBar1->Position = 0;
 ((TMemoryStream*)HttpCli1->RcvdStream)->SetSize(0);
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::HttpCli1DocData(TObject *Sender, Pointer Buffer, int Len)
{
   if (BytesTotal>0)
   {
      BytesReceived += Len;
      ProgressBar1->Position = (BytesReceived/BytesTotal)*100;
      if (BytesReceived==BytesTotal) Sleep(200);
   }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::HttpCli1DocEnd(TObject *Sender)
{
   ProgressBar1->Position = 0;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::MixKeysTooOld()
{
   AnsiString now,last = GetAppIni(MixKeyUpdate, "2002-01-01");
   now = FormatDateTime("yyyy-mm-dd", Date());
   return last!=now;
}
//---------------------------------------------------------------------------
bool __fastcall TRemailerDlg::CpkKeysTooOld()
{
   AnsiString now, last = GetAppIni(CpkKeyUpdate,"2002-01-01");
   now = FormatDateTime("yyyy-mm-dd",Date());
   return last!=now;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::ScrubDocument(AnsiString DocName, TStrings *list)
{
//view(list,"before");
   int pos;
   AnsiString s;
         // handle html
         AnsiString doc=list->Text.UpperCase();
         if (doc.Pos("<HTML>") && doc.Pos("<PRE>") && doc.Pos("</PRE>"))
         {
            while (list->Count && list->Strings[0].UpperCase().Trim()!="<PRE>")
               list->Delete(0); if (list->Count) list->Delete(0);
            while (list->Count && list->Strings[list->Count-1].UpperCase().Trim()!="</PRE>")
               list->Delete(list->Count-1);
            if (list->Count) list->Delete(list->Count-1);
         }
         // get rid of stupid banner  << these may make a bogus report empyt
         if (CurDoc==CListTxt &&
            (pos=list->Text.Pos("$remailer{"))>0 && list->Text.Pos("Last update: ")>pos)
         {
            // dump the banner--bring the caps portion of list to top
            while (list->Count && list->Strings[0].Pos("$remailer{")!=1)
               list->Delete(0);
            // remove all trailing null
            while (list->Count && list->Strings[list->Count-1].IsEmpty())
               list->Delete(list->Count-1);
            // add 1 trailing null
            list->Add("");
            // move all lines above 'Last update: ' to bottom
            if (list->Count && list->Strings[0].Pos("$remailer{")==1)
            while (list->Strings[0].Pos("Last update: ")!=1)
            {
               list->Add(list->Strings[0]);
               list->Delete(0);
            }
            // remove double skipped lines--leave 1 skipped line
            s = list->Text;
            while ((pos=s.Pos("\r\n\r\n\r\n"))>0) s.Delete(pos,2);
            while ((pos=s.Pos("&lt;"))>0) s.Delete(pos,4);
            while ((pos=s.Pos("&gt;"))>0) s.Delete(pos,4);
            list->Text = s;
         }
         // get rid of stupid remailer banner
         if (CurDoc==MListTxt && list->Text.Pos("Last update: "))
         {
            while (list->Count)
            {
               AnsiString line = list->Strings[0];
               if (line.Pos("Last update: ") > 0 || line.Pos("Broken type-I remailer chains:") > 0)
                  break;

               list->Delete(0);
            }
         }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::HttpCli1RequestDone(TObject *Sender,
	THttpRequest RqType, WORD Error)
{
   AnsiString s,TxtFile;
   if (Error)
   {
      PageControl1->ActivePage = ResultsSheet;

      TRichEdit *Doc=CreateDoc("");
      Tabs->AddObject("Error",Doc);
      TabSet->TabIndex = Tabs->Count-1;
      Doc->Lines->Add("HTTP Error = "+AnsiString(Error));
      Doc->Lines->Add("HTTP Reason = "+HttpCli1->ReasonPhrase);
      UrlsBtn->Enabled = true;
      ProxyBtn->Enabled = true;
      TimeOutTimer->Enabled = false;
      return;
   }
   switch (RqType)
   {
      case httpGET:
      {
         TimeOutTimer->Enabled = false;
         HttpCli1->RcvdStream->Seek(0, soFromBeginning);

         TRichEdit *Doc=CreateDocHidden();
         TStringList *doc = new TStringList();
         doc->LoadFromStream(HttpCli1->RcvdStream);
         if (CurDoc==CRingTxt)
         {
            CheckDeleteBarredCypherpunkKeys(doc);
         }
         if (CurDoc==MRingTxt)
         {
            CheckDeleteBarredMixmasterKeys(doc);
         }
         ScrubDocument("",doc);
         Doc->SetSelTextBuf(doc->Text.c_str());

         Tabs->AddObject(CurDoc,Doc);
         Doc->Visible = true;
         Label1->Caption = "";
         AppProcessMessages();
         delete doc;

         int time=(int)Now(); while ((int)Now()>time+100)
            AppProcessMessages();

         AppProcessMessages(); Sleep(500);
         CurBox->Enabled = false;
         AppProcessMessages();

         // auto select keyring for update
         if (CurDoc==MListTxt && !MRingBox->Checked &&
            (MixListNewRemailer(Doc->Lines) || MixKeysTooOld()))
         {
            MRingBox->Checked = true;
            MixSelfUpdate = true;
         }

         if (CurDoc==CListTxt && !CRingBox->Checked &&
            (CpkListNewRemailer(Doc->Lines) || CpkKeysTooOld()))
         {
            CRingBox->Checked = true;
            CpkSelfUpdate = true;
         }

         if (CurDoc==MRingTxt && ErrorCheck->Checked)
         {
            GetDocument(MTyp2Txt);
         }

         if (PageControl1->ActivePage!=ResultsSheet)
             PageControl1->ActivePage =ResultsSheet;
         GetBtnClick();
         break;
      }
   }
}
//---------------------------------------------------------------------------
AnsiString __fastcall TRemailerDlg::GetDefaultURL(AnsiString &DocName)
{
   if (DocName==MListTxt) return MListCBox->Text;
   if (DocName==MRingTxt) return MRingCBox->Text;
   if (DocName==MTyp2Txt) return MTyp2CBox->Text;

   if (!HasCpunk()) return "";

   if (DocName==CListTxt) return CListCBox->Text;
   if (DocName==CRingTxt) return CRingCBox->Text;

   return "";
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::GetAppURLs()
{
   if ((MListURL = GetDefaultURL(MListTxt))=="") {
      MListBox->Checked=false; MListBox->Enabled=false;
   } else MListBox->Enabled=true;

   if ((MRingURL = GetDefaultURL(MRingTxt))=="") {
      MRingBox->Checked=false; MRingBox->Enabled=false;
   } else MRingBox->Enabled=true;

   if ((MTyp2URL = GetDefaultURL(MTyp2Txt))=="" || ErrorCheck->Checked) {
      MTyp2Box->Checked=false; MTyp2Box->Enabled=false;
   } else MTyp2Box->Enabled=true;

   if (!HasCpunk()) return;

   if ((CListURL = GetDefaultURL(CListTxt))=="") {
      CListBox->Checked=false; CListBox->Enabled=false;
   } else CListBox->Enabled=true;

   if ((CRingURL = GetDefaultURL(CRingTxt))=="") {
      CRingBox->Checked=false; CRingBox->Enabled=false;
   } else CRingBox->Enabled=true;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::CancelBtnClick(TObject *Sender)
{
   //Cancelled = true;
   TimeOutTimer->Enabled = false;
   Timer1->Enabled = false;
   Label1->Caption = "";

   TabSet->Enabled = true;
   HelpBtn->Enabled = true;
   OkBtn->Enabled = true;
   Attended = true;

   if (!GetDocs->Enabled)
   {
      GetDocs->Enabled = true;
      //TabSheet1->Enabled = true;
      Panel3->Enabled = true;
      CancelAnyway = true;
      HttpCli1->Abort();
      return;
   }
   ModalResult = mrCancel;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::PageControl1Changing(TObject *Sender,
  bool &AllowChange)
{
   if (PageControl1->ActivePage==ResultsSheet)
      ActiveControl = DocMemo;

   SetRedraw(this,0);
   AppProcessMessages();
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::PageControl1Change(TObject *Sender)
{
   for (int i=0; i<5; i++) AppProcessMessages();
   SetRedraw(this,1);
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::Timer1Timer(TObject *Sender)
{
   if (Label1->Caption=="") Label1->Caption = Cap; else Label1->Caption = "";
   AppProcessMessages();
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::UrlsBtnClick(TObject *Sender)
{
   if (ManageURLs(this,true))
   {
      MListCBox->Clear();
      MRingCBox->Clear();
      MTyp2CBox->Clear();
      if (HasCpunk())
      {
         CListCBox->Clear();
         CRingCBox->Clear();
      }
      AppProcessMessages();
      Sleep(200);
      LoadURLs();
   }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::UrlsBtn2Click(TObject *Sender)
{
   if (ManageURLs(this,false))
   {
      MListCBox->Clear();
      MRingCBox->Clear();
      MTyp2CBox->Clear();
      if (HasCpunk())
      {
         CListCBox->Clear();
         CRingCBox->Clear();
      }
      AppProcessMessages();
      Sleep(200);
      LoadURLs();
   }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::TabSetChange(TObject *Sender, int NewTab,
   bool &AllowChange)
{
   ((TRichEdit*)Tabs->Objects[NewTab])->BringToFront();
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::FormResize(TObject *Sender)
{
   HelpBtn->Left = Width - 92;
   CancelBtn->Left = HelpBtn->Left - 75;
   OkBtn->Left = CancelBtn->Left - 75;
   GetDocs->Left = OkBtn->Left - 75;
   ProgressBar1->Width = GetDocs->Left - 30;
   Label1->Top = Panel1->Height - Label1->Height;
   Label1->Width = Width - 42;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::CRingBoxClick(TObject *Sender)
{
   if (HasCpunk() && CRingBox->Checked) CListBox->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::MRingBoxClick(TObject *Sender)
{
   if (MRingBox->Checked) MListBox->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::HelpBtnClick(TObject *Sender)
{
   AnsiString sections;
   TTabSheet *tab = PageControl1->ActivePage;
   if (tab==TabSheet1)
   {
      sections="Remailer\nRemailerUrl";
      if (ErrorCheck->Checked) sections += "\nRemailerError\nMixmasterErrors";
      OpenHelp(sections,"Remailers & Remailer Documents","Remailer Dialog",NULL);
      return;
   }
   /*if (tab==CpunkSheet)
   {
      sections="Remailer\nRemailerUrl\nTemplates";
      if (ErrorCheck->Checked) sections += "\nRemailerError\nMixmasterErrors";
      OpenHelp(sections,"Remailers & Remailer Documents","Remailer Dialog",NULL);
      return;
   }*/
   if (tab==ResultsSheet)
   {
      HelpBtn->Caption = "Help";
      AnsiString sections =
         s("Remailer_What Is A 'Failed Error Check'\n")+
         s("Remailer_How Do I Fix A Failed Error Check\n")+
         s("Remailer_Sample mlist.txt\n")+
         s("Remailer_Sample type2.list\n")+
         s("Remailer_Sample pubring.mix\n")+
         s("Remailer_Sample rlist.txt\n")+
         s("Remailer_Sample pubring.asc\n")+
         s("RemailerURL_URL Manager");
      OpenHelp2(sections,"I have a 'Failed Error Check', What do I do?",NULL);
      /*sections="RemailerError\nMixmasterErrors";
      if (ErrorCheck->Checked)
      {
         //sections += "\nRemailerError";
         //if (HasMix()) sections += "\nUpdateMixDocs";
         //if (HasCpunk()) sections += "\nUpdateCpkDocs";
         //if (GetAppIni(StatsErrCheck,"").ToIntDef(1)) sections += "\nRemailerClear";
      }
      OpenHelp(sections,"Remailers & Remailer Documents","Remailer Dialog",NULL);
      return;*/
   }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::ProxyBtnClick(TObject *Sender)
{
   if (GetWebProxyInfo(this)) LoadHttpInfo();
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::TimeOutTimerTimer(TObject *Sender)
{
   CurTime++;
   if (CurTime>=TimeOut)
   {
      HttpCli1->Abort();
      TimeOutTimer->Enabled = false;
      Timer1->Enabled = false; Label1->Caption = "";
      AppProcessMessages(); Sleep(300);
      switch (MsgBoxHwnd(Handle,
         s("Hi ! I didn't get '"+CurDoc+"'...the connection timed out. The\n")+
         s("server may be down. I'll continue with any remaining docs.\n")+
         s("You may need to try another URL for "+CurDoc+".\n\n")+
         s("Would you like to try again before continuing?"),
         s(CurDoc.UpperCase()+" Timed Out"),
         MB_YESNOCANCEL))
      {
         case IDCANCEL: CancelBtn->Click(); break;
         case IDNO : CurBox->Enabled = false;
         case IDYES: GetBtnClick();
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::DeleteBtnClick(TObject *Sender)
{
   TStringList *files = new TStringList();
   files->Text = ClearChosenDocuments(this);
   for (int i=0; i<files->Count; i++)
   {
      AnsiString file=files->Strings[i];
      DeleteFile(file.c_str());

      if (file==mringtxt && MixKeyWin2)
      {
         Sleep(100);
         MixKeyWin2->keys->modified = false;
         MixKeyWin2->Close();
      }
      if (HasCpunk() && file==cringtxt && CpkKeyWin2)
      {
         Sleep(100);
         CpkKeyWin2->keys->modified = false;
         CpkKeyWin2->Close();
      }
   }
   delete files;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::ErrorCheckClick(TObject *Sender)
{
  if (ErrorCheck->Checked)
  {
     MTyp2CBox->Visible = false;
     MTyp2Box->Checked = false; MTyp2Box->Enabled = false;
     //CTyp1Box->Visible = true;
  }
  else
  {
     MTyp2CBox->Visible = true;
     //CTyp1Box->Visible = false;
     MTyp2Box->Enabled = true;
  }
  //ErrorCheck2->Checked = ErrorCheck->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::HttpCli1HeaderEnd(TObject *Sender)
{
   for (int i=0; i<HttpCli1->RcvdHeader->Count; i++)
   {
      AnsiString hdr = HttpCli1->RcvdHeader->Strings[i];
      if (hdr.Pos("Content-Length: ")==1)
      {
         BytesTotal = hdr.SubString(17,16).ToIntDef(0);
       ((TMemoryStream*)HttpCli1->RcvdStream)->SetSize(BytesTotal);
         break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::CListBoxClick(TObject *Sender)
{
   if (!CListBox->Checked && CRingBox->Visible) CRingBox->Checked = false;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::MListBoxClick(TObject *Sender)
{
   if (MListBox->Checked) CListBox->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TRemailerDlg::DocErrorBtnClick(TObject *Sender)
{
   HelpBtn->Caption = "Help";
   AnsiString sections =
      s("Remailer_What Is A 'Failed Error Check'\n")+
      s("Remailer_How Do I Fix A Failed Error Check\n")+
      s("Remailer_Sample mlist.txt\n")+
      s("Remailer_Sample type2.list\n")+
      s("Remailer_Sample pubring.mix\n")+
      s("Remailer_Sample rlist.txt\n")+
      s("Remailer_Sample pubring.asc\n")+
      s("RemailerURL_URL Manager");
   OpenHelp2(sections,"I have a 'Failed Error Check', What do I do?",NULL);
}
//---------------------------------------------------------------------------
